//
//  Mesure de tension
//
//
// Auteur : www.thyzoon.fr 
//
// 2011-2013
//

//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "Tension_Lipo.h"

TensionLipo::TensionLipo() {
}

void TensionLipo::begin(){
   analogReference(DEFAULT); // reference alim. 5V
   digitalWrite(A0, LOW); 
   pinMode(A0, INPUT);
   digitalWrite(A1, LOW); 
   pinMode(A1, INPUT);
   digitalWrite(A2, LOW); 
   pinMode(A2, INPUT);
   digitalWrite(A3, LOW); 
   pinMode(A3, INPUT);   
   digitalWrite(A4, LOW); 
   pinMode(A4, INPUT);    
   digitalWrite(A5, LOW); 
   pinMode(A5, INPUT); 
}

float TensionLipo::Tension_Lipo1 (){
  static float MV1 = 0.0; //4.20;
  static float MV2 = 0.0;
  float MV3;
  float val;
  
  MV3 =  mesure(A0)* COEF_LIPO1;
  val = (MV1 + MV2 + MV3) / 3; // filtrage (moyenne de 3 echantillons)
  MV1 = MV2; // décalage
  MV2 = MV3;
  
  return val;
}

float TensionLipo::Tension_Lipo2 (){
  static float MV1 = 0.0;
  static float MV2 = 0.0;
  float MV3;
  float val;
  
  MV3 =  mesure(A1)* COEF_LIPO2;
  val = (MV1 + MV2 + MV3) / 3; // filtrage (moyenne de 3 echantillons)
  MV1 = MV2; // décalage
  MV2 = MV3;
  
  return val;  
}

float TensionLipo::Tension_Lipo3 (){
  static float MV1 = 0.0;
  static float MV2 = 0.0;
  float MV3;
  float val;
  
  MV3 =  mesure(A2)* COEF_LIPO3;
  val = (MV1 + MV2 + MV3) / 3; // filtrage (moyenne de 3 echantillons)
  MV1 = MV2; // décalage
  MV2 = MV3;
  
  return val;  
}

float TensionLipo::Tension_Lipo4 (){
  static float MV1 = 0.0;
  static float MV2 = 0.0;
  float MV3;
  float val;
  
  MV3 =  mesure(A3)* COEF_LIPO4;
  val = (MV1 + MV2 + MV3) / 3; // filtrage (moyenne de 3 echantillons)
  MV1 = MV2; // décalage
  MV2 = MV3;
  
  return val;  
}

float TensionLipo::Tension_Lipo5 (){
  static float MV1 = 0.0;
  static float MV2 = 0.0;
  float MV3;
  float val;
  
  MV3 =  mesure(A4)* COEF_LIPO5;
  val = (MV1 + MV2 + MV3) / 3; // filtrage (moyenne de 3 echantillons)
  MV1 = MV2; // décalage
  MV2 = MV3;
  
  return val;
}

float TensionLipo::Tension_Lipo6 (){
  static float MV1 = 0.0;
  static float MV2 = 0.0;
  float MV3;
  float val;
  
  MV3 =  mesure(A5)* COEF_LIPO6;
  val = (MV1 + MV2 + MV3) / 3; // filtrage (moyenne de 3 echantillons)
  MV1 = MV2; // décalage
  MV2 = MV3;
  
  return val;    
}

// Detection du nombre d'elements du pack
uint8_t TensionLipo::Detect () {
  float val;
  
  uint8_t nb_Lipo = 2;  // Minimum : 2S, valeur par défaut.
  
  // On commence par des lectures bidons pour initialiser les filtres
  for (uint8_t index = 0; index < 2; index++)
  {  
    val = TensionLipo::Tension_Lipo1 (); // Mesure sur element 1
    val = TensionLipo::Tension_Lipo2 (); // Mesure sur element 2
    val = TensionLipo::Tension_Lipo3 (); // Mesure sur element 3
    val = TensionLipo::Tension_Lipo4 (); // Mesure sur element 4    
    val = TensionLipo::Tension_Lipo5 (); // Mesure sur element 5   
    val = TensionLipo::Tension_Lipo6 (); // Mesure sur element 6
  }  

  val = TensionLipo::Tension_Lipo3 (); // Mesure sur element 3
  if (val > 8.0) {         // > 8V  
      nb_Lipo = 3;}
      
  val = TensionLipo::Tension_Lipo4 (); // Mesure sur element 4
  if (val > 11.0) {         // > 11V  
      nb_Lipo = 4;}  

  val = TensionLipo::Tension_Lipo5 (); // Mesure sur element 5
  if (val > 14.0) {         // > 14V  
      nb_Lipo = 5;}  

  val = TensionLipo::Tension_Lipo6 (); // Mesure sur element 6
  if (val > 17.0) {         // > 17V  
      nb_Lipo = 6;}        
      
  return nb_Lipo;  
}  

// Recherche element le plus faible
float TensionLipo::Lipo_Mini (uint8_t nb_Lipo,float L1,float L2,float L3,float L4,float L5,float L6){
  float mini = 0.0;
  
  mini = L1;
  if (L2 < mini) {
	mini = L2;
  }

  if ((nb_Lipo >= 3) && (L3 < mini)){
    mini = L3;
  }

  if ((nb_Lipo >= 4) && (L4 < mini)){
    mini = L4;
  }
  
  if ((nb_Lipo >= 5) && (L5 < mini)){
    mini = L5;
  }  
  
  if ((nb_Lipo == 6) && (L6 < mini)){
    mini = L6;
  }  
  
  return mini;
}

// Mesure tension (CAN)
uint16_t TensionLipo::mesure(uint8_t pin){
  uint16_t val = analogRead(pin); 
  return val;
}


